#!/bin/sh 

# bb-local.sh
# BIG BROTHER MONITORING SCRIPT
# Sean MacGuire - The MacLawran Group Inc.
# Version 1.06
# Dec 30, 1997
#
# This program is Copyright (c) 1997 
# The MacLawran Group Inc.
# All Rights Reserved
#
# For each system we monitor we watch
#	disk	USES df 
#	cpu	USES uptime
#	procs	CHECKS A LIST
# 	msgs	LOOKS FOR WARNINGS
#
# For the network we care about
#	telnet	USES telnet and checks the results
#	http	USES lynx to check for valid output
#
# THEN SEND THE RESULTS TO THE DISPLAY SERVER USING bb

# echo "***** BBHOME IS SET TO $BBHOME"
# echo "***** BBTMP IS SET TO $BBTMP"

if test ! "$BBTMP"                      # GET DEFINITIONS IF NEEDED
then
	 # echo "*** LOADING BBDEF ***"
        . $BBHOME/etc/bbdef.sh          # INCLUDE STANDARD DEFINITIONS
fi

#
#=====================================================================
# DISK TEST
#=====================================================================
#
# echo "*** DISK TEST 1 ***"
COLOR="green"
LINE="
"

$DF | $GREP $DFUSE | $GREP -v $DFEXCLUDE | $SORT +${DFSORT} > $BBHOME/tmp/DFOUT
cat $BBHOME/tmp/DFOUT | {
while read line
do
    set $line
    DISKNAME=$6
    DISKDEV=$1
    DISKPCT=`echo $5 | $SED "s/%//"`

    # echo "*** DISK TEST 2 ***"
    #
    # SUPPORT FOR NON-STANDARD df'S
    # Richard Dansereau <dansereau.richard@regis.uqam.ca>
    #
    case "$DFOUTPUT"
    in
        SCO3)   DISKNAME=$1
                DISKPCT=`echo $6 | $SED "s/%//"`
		DISKDEV=$7	# Is this true ?
		;;
    esac

    # echo "$DISKNAME $DISKPCT"

	DFWARNVAL=$DFWARN               # USE DEFAULT WARN VALUE
	DFPANICVAL=$DFPANIC             # USE DEFAULT PANIC VALUE

	#
	# NEW IN 1.08a
	# Robert Andre Croteau's custom disk file
	# IF WE HAVE A CUSTOM PARTITION FILE, USE IT.
	# DEFAULTS TO etc/bb-dftab
	# FORMAT:	partition:warn%:panic%
	# I.E.:		/usr:96:99
	#
	if test -f "${BBDFTAB}"
	then
		# echo "*** CHECKING FOR PARTITION $DISKNAME IN $BBDFTAB"
		DISKLINE=`$GREP "^${DISKNAME}:" $BBDFTAB`
		# echo "*** DISKLINE: $DISKLINE"
		if test "$DISKLINE" != ""
		then
			# echo "*** DISKLINE NOT NULL"
			set `echo $DISKLINE | $SED "s/:/ /g"`
        		DFWARNVAL=$2            # GET CURRENT FS WARNING LEVEL
        		DFPANICVAL=$3           # GET CURRENT FS PANIC LEVEL
			# echo "*** WARN AT $DFWARNVAL - PANIC AT $DFPANICVAL"
		fi
	fi

    	MARK=""
    	if test "$DISKPCT" -ge "$DFWARNVAL"         # ABOVE WARNING LEVEL
    	then
        	if test "$DISKPCT" -ge "$DFPANICVAL"    # ABOVE PANIC, RED ALERT !!!
        	then
            		COLOR="red"
            		MARK="PANIC"
	    		MARKLEVEL=$DFPANICVAL
	    		HIGHMARK="PANIC"
        	else
            		if test "$COLOR" = "green"          # ONLY IF NOT ALREADY SET
            		then
                		COLOR="yellow"
	        		HIGHMARK="WARNING"
            		fi
            		MARK="WARNING"
	    		MARKLEVEL=$DFWARNVAL
        	fi
        	LINE="${LINE}
${DISKNAME} (${DISKPCT}%) has reached the defined disk space ${MARK} level (${MARKLEVEL}%)"
    	fi
done

#
# NOW SEND THIS INFORMATION TO THE BIG BROTHER DISPLAY UNIT
#
if test "$COLOR" = "red"
then
    if test "$DFPAGE" = "Y"             # CALL THE PAGER
    then
        $BB $BBPAGE "page ${DISKERR}${MACHIP} `echo $LINE`"
    fi
fi

if [ "$COLOR" = "green" ]
then
    LINE="status $MACHINE.disk $COLOR `date` - Disk partitions on $MACHINE OK

`cat $BBHOME/tmp/DFOUT`"
else
    LINE="status $MACHINE.disk $COLOR `date` - Disk on $MACHINE at $HIGHMARK level $LINE

`cat $BBHOME/tmp/DFOUT`"
fi

$BB $BBDISP "$LINE"

$RM -f $BBHOME/tmp/DFOUT

}
#
#=====================================================================
# CPU TEST AND UPTIME WARNINGS
# AUTOMATIC WARNING IF THE SYSTEM IS UP FOR LESS THAN 1 HOUR
#=====================================================================
#
# echo "*** CPU TEST ***"
set `$UPTIME`

echo $3 | $GREP "," > /dev/null 2>&1
if test "$?" = 0
then
	UP=`echo $3 | $SED "s/,//"`
else
	UP=`echo $3 $4 | $SED "s/,//"`
fi

shift;shift;shift;shift;

ARG=`expr $# - 2`			# 5 MINUTE LOAD AVERAGE
COUNT=1
while test "$COUNT" -le "$ARG"
do
	shift;
	COUNT=`expr $COUNT + 1`
done

AVG=`echo $1 | $SED "s/\,//g" | $SED "s/\.//" `
AVG=`expr $AVG + 0`
# echo "LOAD AVERAGE: $AVG"

LINE="LOAD AVG on $MACHINE is ${AVG}"

if test "$AVG" -ge "$CPUPANIC"
then
	COLOR="red"
	if test "$CPUPAGE" = "Y"         # CALL THE PAGER
	then
		$BB $BBPAGE "page ${CPUERR}${MACHIP} $LINE (paging)"
	fi
elif test "$AVG" -ge "$CPUWARN"
then
	COLOR="yellow"
else	
	COLOR="green"
fi

# echo "*** PS COMMAND $PS ***"

$PS > $BBTMP/bb.$$			# GET A PS LISTING
P=`$CAT $BBTMP/bb.$$ | $WC`		# NUMBER OF PROCESSES
P=`echo $P`
U=`$WHO | $WC`				# GET NUMBER OF USERS
U=`echo $U`

#
# AUTOMATIC WARNING IF A SYSTEM HAS BEEN UP LESS THAN AN HOUR
# THIS WILL HELP US NOTICE SPONTANEOUS REBOOTS!
#
echo $UP | grep "min" > /dev/null 2>&1
if test "$?" = "0"
then
	COLOR="yellow"
fi

#
# NOW SEND THIS INFORMATION TO THE BIG BROTHER DISPLAY UNIT
#
$BB $BBDISP "status $MACHINE.cpu $COLOR `date` up: $UP, $U users, $P procs, load=${AVG}"

#
#=====================================================================
# PROCESSES THAT MUST EXIST
#=====================================================================
#
# echo "*** PROCESSES TEST ***"

LINE=""
COLOR="green"

for proc in $PROCS $PAGEPROC		# SMM CHECK BOTH
do
	$GREP "$proc" $BBTMP/bb.$$ | $GREP -v grep > /dev/null 2>&1
	if test "$?" != "0"
	then
		LINE="$LINE - $proc DOWN"
		#
		# NOW SEE IF THIS IS A PROCESS WE SHOULD PAGE ON...
		# BUT ONLY IF A PAGER IS ACTUALLY DEFINED...
		#
		echo "$PAGEPROC" | $GREP "$proc" > /dev/null 2>&1
		if test "$?" = "0"
		then
			if test "$PAGER" != ""
			then
				$BB $BBPAGE "page ${PROCERR}${MACHIP} ${MACHINE} $proc DOWN"
				LINE="${LINE} (paging)"
			fi
			COLOR="red"
		fi	
	else
		LINE="$LINE - $proc OK"
	fi
done

#
# IF SOMETHING IS DOWN, AND IT'S NOT CRITICAL, JUST WARN
#
echo "$LINE" | $GREP "DOWN" > /dev/null 2>&1
if test "$?" = "0"				# SOMETHING IS DOWN
then
	if test "$COLOR" = "green"		# WASN'T A PANIC
	then
		COLOR="yellow"			# SO JUST WARN
	fi
fi
#
# NOW SEND THIS INFORMATION TO THE BIG BROTHER DISPLAY UNIT
#
$BB $BBDISP "status $MACHINE.procs $COLOR `date` $LINE"
$RM -f $BBTMP/bb.$$

#
#=====================================================================
# NOTIFY ON BAD MESSAGES
#=====================================================================
#
# echo "*** MESSAGES TEST ***"

COLOR="green"

for word in $MSGS
do
	LINE=`$GREP "$word" $MSGFILE`
	if test "$LINE" != ""			# FOUND A BAD WORD
	then
		echo "$PAGEMSG" | $GREP "$word" > /dev/null 2>&1
		if test "$?" = "0"
		then
			COLOR="red"
			if test "$PAGER" != ""
			then
				LINE="$MACHINE bad message: ${LINE}"
				$BB $BBPAGE "page ${MSGERR}${MACHIP} $LINE"
				LINE="$LINE (paging)"
			fi
			$BB $BBDISP "status $MACHINE.msgs $COLOR `date` $LINE"
			break;			# ENOUGH ALREADY...
		else
			if test "$COLOR" = "green"
			then
				COLOR="yellow"
				$BB $BBDISP "status $MACHINE.msgs $COLOR `date` $LINE"
			fi
		fi
	fi
done

if test "$COLOR" = "green"
then
	$BB $BBDISP "status $MACHINE.msgs $COLOR `date` No bad messages"
fi

