#!/bin/sh

# bb-network.sh
# BIG BROTHER NETWORK MONITORING SCRIPT
# Sean MacGuire - The MacLawran Group Inc.
# Version 1.07c
# Aug 04, 1998
#
# This program is Copyright (c) 1997, 1998
# The MacLawran Group Inc.
# All Rights Reserved
#
# SEND THE TEST RESULTS TO THE DISPLAY SERVER USING bb

# echo "***** BBHOME IS SET TO $BBHOME IN bb-net"
if test ! "$LYNX"			# GET DEFINITIONS IF NEEDED
then
	. $BBHOME/etc/bbdef.sh		# INCLUDE STANDARD DEFINITIONS
fi

#
# SMM - NEW FOR 1.05
# ALLOW TESTING OF BANKS OF CONNECTIONS ONLY
# FORMAT OF THE LINE IS
# dialup name IP-ADDRESS COUNT
# dialup modem-bank1 132.220.50.1 16
#
$GREP "^dialup" $BBHOSTS |
while read line
do
	set $line
	if test "$#" != "4"
	then
		echo "MISSING ARGUMENTS FOR dialup"
		echo "Line format is:"
		echo "dialup pool-name start-ip count"
		exit 1
	fi
	NAME="$2"
	TTL="$4"
	set `echo $3 | $SED "s/\./ /g"`
	BASE="$1.$2.$3"
	START="$4"
	END=`expr $START + $TTL - 1`
	# echo "NAME IS $NAME"
	# echo "TTL IS $TTL"
	# echo "BASE IS $BASE"
	# echo "START IS $START"
	# echo "END IS $END"
	LINE="FROM $BASE.$START TO $END DATA "
	while test "$START" -le "$END"
	do
		TEST="${BASE}.$START"
		# echo $TEST
		$PING $PINGPAR1 $TEST $PINGPAR2 > $BBTMP/BBPING 2>&1
		$GREP "bytes from" $BBTMP/BBPING > /dev/null 2>&1
		if test $? != "0"
		then
			LINE="$LINE clear "
		else
			LINE="$LINE green "
		fi
		START=`expr $START + 1`
	done
	# echo "LINE IS $LINE"
	$BB $BBDISP "status dialup.${NAME} green `date` $LINE" 
done

$GREP "^[0-9]" $BBHOSTS |
while read line
do
	set $line
	HOSTIP=$1			# HOST IP ADDRESS
	IPADDR=`echo $1 | $SED "s/\.//g"`
	#
	# SMM - NEW FOR 1.07c
	# IMPLICIT SUPPORT FOR dhcp STATIONS WHEN LISTED AS 0.0.0.0
	# BUT THE PING TEST MUST BE AUTOMATICALLY DISABLED (noping)
	#
	if test `expr $IPADDR + 0` = 0
	then
		line="$line noping"	# DISABLE THE PING TEST
	fi
	FULLNAME=$2
	#
	# SMM - NEW FOR 1.04
	# USE FULLY QUALIFIED MACHINE NAMES?
	# DOT -> COMMA -> DOT FOR WEB DISPLAY
	#
	if test "$FQDN" = "TRUE"
	then
		set `echo $FULLNAME | $SED "s/\./,/g"`
	else
		set `echo $FULLNAME | $SED "s/\./ /"`
	fi
	MACHINE=$1
	#
	#================================================
	# CONNECTIVITY TEST
	# Added support for non-standard pings. gunnar@bitcon.no
	#
	# SMM: PING BY IP ADDRESS INSTEAD OF BY NAME. (v1.04a)
	# SMM: TRY THREE TIMES BEFORE PAGING (v1.06b)
	# 	BB SHOULD NOT BE A 3AM WAKE UP CALL...
	#================================================

	#
	# SMM 1.07
	# TO PING OR NOT TO PING
	# FIXED FALSE POSITIVES THANKS TO consult@netcom.com
	#
	echo "$line" | $GREP "noping" > /dev/null 2>&1
	ERR=$?
	if test $ERR = "0"
	then
		COLOR="clear"			# PING TEST DISABLED BY REQUEST
		LINE="Ping test disabled by request (noping)"
	else
		COLOR="green"			# INNOCENT UNTIL PROVEN GUILTY...
		LINE="Connection OK"
		$PING $PINGPAR1 $HOSTIP $PINGPAR2 > $BBTMP/BBPING 2>&1
		ERR=$?
		if test $ERR = "0"		# PASSED THE FIRST TEST...
		then
			$GREP "bytes from" $BBTMP/BBPING > /dev/null 2>&1
			ERR="$?"
		fi
	fi
	if test $ERR != "0"
	then
		sleep 20		# PAUSE... AND TRY AGAIN...
		$PING $PINGPAR1 $HOSTIP $PINGPAR2 > $BBTMP/BBPING 2>&1
		ERR=$?
		if test $ERR = "0"		# PASSED THE FIRST TEST...
		then
			$GREP "bytes from" $BBTMP/BBPING > /dev/null 2>&1
			ERR=$?
		fi
		if test $ERR != "0"
		then
			echo "$line" | $GREP "dialup" > /dev/null 2>&1
			if test $? = "0"
			then
				COLOR="clear"
				LINE="bb-network.sh: $FULLNAME offline (dialup)"
			else		# NON-DIALUP GETS 3rd STRIKE...
				COLOR="red"
				LINE="bb-network.sh: Can't ping $FULLNAME..."
				sleep 30

				$PING $PINGPAR1 $HOSTIP $PINGPAR2 > $BBTMP/BBPING 2>&1
				ERR=$?
				if test $ERR = "0"	# PASSED THE FIRST TEST...
				then
					$GREP "bytes from" $BBTMP/BBPING >/dev/null 2>&1
					ERR=$?
				fi
				if test $ERR != "0"	# THREE STRIKES...
				then
					echo "$NONETPAGE" | $GREP "ping" >/dev/null 2>&1
					if test $? = "1"	# NOT FOUND - PAGE
					then
						$BB $BBPAGE "page ${TELNETERR}${IPADDR} ${LINE}"
						LINE="$LINE (paging)"
					fi
				else	# OK ON THIRD TRY... CAME BACK...
					COLOR="yellow"	
					LINE="Connection OK on 3rd try..."
				fi
			fi
		fi
	fi
	$BB $BBDISP "status ${MACHINE}.conn $COLOR `date` $LINE" 
	$RM -f $BBTMP/BBPING		# GET RID OF THE LEFTOVERS

	#
	# THIS IS NEW WITH 1.04... TEST FOR OTHER SERVICES HERE...
	# SEND THE RESULTS TO THE USUAL PLACE...
	#
	set $line			# GET ALL THE LINE ARGS
	shift; shift;			# SKIP THE IP-ADDR AND NAME

	while test "$1"			# AS LONG AS WE HAVE ARGS
	do
		line="$*"		# SAVE THIS..

		case $1 
		in

		#
		# Add telnet test Tony DeJoie (tony@bellcore.com)
		# Add ssh test Kim Shrier (kim@tinker.com)
		#
		ssh* | telnet* | nntp* | ftp* | pop* | smtp* )	# OUR SERVICES
			#
			# OK... GET THE SERVICE NUMBER OUT OF /etc/services
			# CAN BE SEPERATED BY SPACES/TABS... (v1.04a)
			#
			SVCNAME=$1
			SVC=""
			#
			# SMM 1.07a - 21 JUN 98
			# A LITTLE FRIENDLY CHECK HERE WILL SAVE LOTS OF GRIEF
			#
			SVCLINE=`$GREP "^${1}[ 	]" /etc/services | $GREP "/tcp"`
			if test "$SVCLINE" = ""	# HOUSTON, WE HAVE A PROBLEM
			then
				echo "bb-network.sh: ERROR on bb-hosts line: $FULLNAME"
				echo "$SVCNAME NOT in /etc/services (CHECK SPELLING)"
				set $line
				shift;
				continue;
			fi

			set $SVCLINE

			while test "$SVC" = ""
			do
				echo $1 | $GREP "/tcp" > /dev/null
				if test "$?" = "0"
				then
					SVC=`echo $1 | $SED "s/\/tcp//"`
					#echo "FOUND SERVICE: $SVCNAME: $SVC"
				fi
				shift;
			done
			# echo "TESTING SERVICE $SVC"
			OUTPUT=`$BBHOME/bin/bbnet ${FULLNAME}:${SVC}` 
			if test "$?" != "0"
			then
				# echo "SERVICE IS BROKEN"
				#
				# IF IT'S BROKEN, THEN SCREAM IF WE HAVE TO
				#
                		LINE="SERVER - $SVCNAME DOWN"
				#
				# SMM NEW IN 1.07b
				# CORRECT TEST FOR NETWORK SERVICE PAGING
				# THANKS TO: Patrick Hopewell <p.hopewell@utoronto.ca>
				#
				echo "$NONETPAGE" | $GREP "$SVCNAME" > /dev/null 2>&1
				if test $? = "1"	# NOT FOUND - PAGE
                		then
					echo "$line" | $GREP "dialup" > /dev/null 2>&1
					if test $? = "0"
					then
                                		LINE="${LINE} (no page on dialup)"

                        		else
						if test "$PAGER" != ""
                        			then
							# FIX BUG:MACHIP -> IPADDR
							# Bob Teeter <bobt@swdata.com>
		      					$BB $BBPAGE "page ${NETERR}${SVC}${IPADDR} ${MACHINE} $SVCNAME DOWN"
                                			LINE="${LINE} (paging)"
						fi
                        		fi
				fi
				#
				# NO MORE RED DOTS FOR DIALUP NETWORK STUFF
				#
				echo "$line" | $GREP "dialup" > /dev/null 2>&1
				if test $? = "0"
				then
                                	COLOR="clear"
				else
                        		COLOR="red"
				fi
			else
				LINE="$SVCNAME OK: $OUTPUT"
				COLOR="green"
				# echo $OUTPUT
			fi
			$BB $BBDISP "status ${MACHINE}.${SVCNAME} $COLOR `date` $LINE" 
			;;

		#
		# NEW IN 1.06C 
		# DNS TESTING FROM Frank T. Kujawski <frank@bluegrass.net>
		#
		dns )	# echo "looking for domain server"
			$NSLOOKUP ${FULLNAME} ${HOSTIP} > /dev/null
			if test "$?" != "0"
			then
				COLOR="red"
				LINE="DNS Server on ${FULLNAME} down"
				echo "$NONETPAGE" | $GREP "$SVCNAME" > /dev/null 2>&1
				if test $? = "1"	# NOT FOUND - PAGE
				then
					$BB $BBPAGE "page ${DNSERR}${IPADDR} $LINE"
				fi
                                LINE="$LINE (paging)"
			else
				COLOR="green"
				LINE="DNS Server OK"
			fi

			$BB $BBDISP "status ${MACHINE}.dns $COLOR `date` $LINE"

                        ;;

		http* )	# echo "DOING HTTP TEST..."
			HTTP=$1
			$LYNX $HTTP > $BBTMP/HTTP 2>&1
			if test "$?" != 0
			then
				LINE="bb-network.sh: Lost HTTP connection to $HTTP"
				echo "$NONETPAGE" | $GREP "http" > /dev/null 2>&1
				if test $? = "1"	# NOT FOUND - PAGE
				then
					echo "$line" | $GREP "dialup" > /dev/null 2>&1
					if test $? = "0"
					then
                                		LINE="${LINE} (no page on dialup)"
					else
						$BB $BBPAGE "page ${HTTPERR}${IPADDR} $LINE"
						LINE="$LINE - Paging..."
					fi
				fi
				$BB $BBDISP "status ${MACHINE}.http red `date` $LINE"
			else
			#
			# CHECK FOR A SERVER ERROR HIDING HERE... CHECK FOR IT
			#

			#
			# THANKS TO: Douwe Dijkstra <D.Dijkstra@Twinfo.nl>
			# Version 1.04b
			#
				read HTTPVER RC STATUSMESSAGE < $BBTMP/HTTP
				case $RC in
    				'200'|'401'|'302')
         				LINE="Server OK: "
         				COLOR="green"
         				;;
    				*)
         				LINE="Server **NOT** OK: "
         				COLOR="yellow"
         				;;
				esac
				#
				# SMM 1.07c - SEND http HEADERS ALONG
				#
				$BB $BBDISP "status ${MACHINE}.http $COLOR `date` $LINE `cat $BBTMP/HTTP` "
				$RM -f $BBTMP/HTTP
				HTTP=""
			fi
			;;
		esac
		set $line		# RESTORE...
		shift;			# GET NEXT ARGUMENT
	done
done
