#!/bin/sh

#
# bb-page.sh
# BIG BROTHER PAGER SCRIPT
# Sean MacGuire - The MacLawran Group Inc.
# Version 1.07c
# Aug 01, 1998
#
# This program is Copyright (c) 1997, 1998
# The MacLawran Group Inc.
# All Rights Reserved
#
# Format: bb-page <NUMERIC-MESSAGE> <ALPHA-MESSAGE>
#
# Modified to be generic front-end for all paging systems
# and gets all info out of etc/bbdef.sh file
#

#
# Changed 970111 by Jacob Lundqvist jaclu@ibk.se
#
#  Included working SMS and improved NUMERIC paging
#  
#  Now kermit attempts are queued!
#

if test "$#" = "1"	# CALLED AS A SINGLE ARE FROM bbd
then
	set $1		# PUT INTO THE ENVIRONMENT
fi

if test "$#" -lt "2"	# IF THERE AREN'T AT LEAST TWO ARGUMENTS
then
	echo "bb-page:	Incorrect number of arguments"
	echo "Format:	bb-page <NUMERIC-MESSAGE> <ALPHA-MESSAGE>"
	exit 1
fi

BBNUMERIC=`echo $1`	# THE NUMERIC MESSAGE
shift;
BBALPHA="$*"		# THE ALPHA MESSAGE

# THE TTY LINE TO USE
# THE LOCATION OF KERMIT
# THE NUMERIC PREFIX TO DIAL

if test ! "$BBTMP" 			# GET DEFINITIONS IF NEEDED
then
	. $BBHOME/etc/bbdef.sh          # INCLUDE STANDARD DEFINITIONS
fi

#
# SMM: NEW IN 1.05 - PAGE ONLY IF THE REQUIRED PAGERDELAY
# HAS PASSED ALREADY...
#
$BBHOME/bin/touchtime 0  ${BBTMP}/timenow
$FIND $BBTMP -name nextpage -newer ${BBTMP}/timenow -print | $GREP nextpage > /dev/null
if test $? = 0
then
	# echo "Page delay not expired... page cancelled..."
	exit 1
fi

#
# IT'S OK TO PAGE, SO SET THE TIME WE CAN SEND THE NEXT ONE...
#
$BBHOME/bin/touchtime $PAGERDELAY ${BBTMP}/nextpage

#
# NOW WE DECIDE HOW TO HANDLE THINGS
# PAGER CONTAINS A LIST OF PAGERS TO CALL, ARGS AS FOLLOWS
#
#	NUMBER 		SEND DUMB OLDNUMERIC MESSAGE
#	XXX@YYYY	SEND e-mail TO THIS ADDRESS
#	sms-NUMBER 	SEND sms MESSAGE
#	alpha-NUMBER	SEND alpha MESSAGE
#
# EXAMPLE:
#
# 	PAGER="123123 sean@MacLawran.ca sms-123123"
# 	THIS SHOULD SEND A NUMERIC PAGE TO 123123, 
#	MAIL TO sean@MacLawran.ca
# 	AND AN SMS-PAGE TO 123123
#
# SMM 1.07c: CHANGED THE if STATEMENTS TO A case STATEMENT
#	SHOULD BE MUCH EASIER TO MODIFY NOW.
#

for page in `echo $PAGER`		# FOR EACH PAGER ARGUMENT...
do

	case $page in

	*@* )		# echo "EMAIL"		# NOTIFY BY e-mail
			echo $BBALPHA | $MAIL "!BB! $BBALPHA" $page
			;;

	[0-9]* )	# echo "NUMERIC PAGING"
			$BBHOME/bin/sendmsg $BBHOME/etc/numeric.scr ${PREFIX}$page $BBNUMERIC
			;;

	sms-* )		# echo "SMS"
			set `echo $page | sed "s/-/ /"`
			NUMBER=$2
			#
			# Message must be stored in a file, 
			# for the sms kermit script
			#
			SMSFILE=${BBTMP}/sms-$$
			echo "$BBALPHA" > $SMSFILE
			#
			# Note!!!
			# =======
			# If you use a dialing prefix, it must be manually set in
			# sms.scr
			# Here we only send the numbers to page, not the paging
			# server
			#
			$BBHOME/bin/sendmsg $BBHOME/etc/sms.scr $SMSFILE $NUMBER
			$RM -f $SMSFILE
			;;

	alpha-* )	# echo "ALPHA PAGING"
			#
			# SUPPORT FOR ALPHA PAGING OR WHATEVER...
			# THIS ISN'T IMPLEMENTED YET!
			# SUGGESTIONS ANYONE?
			#
			set `echo $page | sed "s/-/ /"`
			NUMBER=$2
			;;

	* )		# echo "DEFAULT"
			;;
	esac
done

exit $?
